using System;
using System.Web;
using System.IO;
using System.Web.Caching;

using CMS.FileManager;
using CMS.WorkflowEngine;
using CMS.TreeEngine;
using CMS.GlobalHelper;
using CMS.CMSHelper;
using CMS.DataEngine;
using CMS.SiteProvider;
using CMS.SettingsProvider;
using CMS.PortalEngine;
using CMS.URLRewritingEngine;
using CMS.WebAnalytics;
using CMS.UIControls;

public partial class CMSPages_GetFile : GetFilePage
{
    #region "Advanced settings"

    /// <summary>
    /// Set to false to disable the client caching
    /// </summary>
    protected bool useClientCache = true;

    /// <summary>
    /// Set to 0 if you do not wish to cache large files
    /// </summary>
    protected int largeFilesCacheMinutes = 1;

    #endregion


    #region "Variables"

    protected CMSOutputFile outputFile = null;

    protected AttachmentManager mAttachmentManager = null;
    protected TreeProvider mTreeProvider = null;
    protected GeneralConnection mConnection = null;

    protected TreeNode node = null;
    protected PageInfo pi = null;

    #endregion


    #region "Properties"

    /// <summary>
    /// Attachment manager
    /// </summary>
    public AttachmentManager AttachmentManager
    {
        get
        {
            if (mAttachmentManager == null)
            {
                mAttachmentManager = new AttachmentManager(this.Connection);
            }

            return mAttachmentManager;
        }
    }


    /// <summary>
    /// Connection
    /// </summary>
    public GeneralConnection Connection
    {
        get
        {
            if (mConnection == null)
            {
                mConnection = ConnectionHelper.GetConnection();
            }

            return mConnection;
        }
    }


    /// <summary>
    /// Tree provider
    /// </summary>
    public TreeProvider TreeProvider
    {
        get
        {
            if (mTreeProvider == null)
            {
                mTreeProvider = new TreeProvider(this.CurrentUser, this.Connection);
            }

            return mTreeProvider;
        }
    }

    #endregion


    protected void Page_Load(object sender, EventArgs e)
    {
        ConnectionHelper.SetContext("GetFile");

        // Check the site
        if (CurrentSiteName == "")
        {
            throw new Exception("[GetFile.aspx]: Site not running.");
        }

        // Set campaign
        if (this.ViewMode == ViewModeEnum.LiveSite)
        {
            // Store campaign name if present
            string campaign = AnalyticsHelper.CurrentCampaign(CurrentSiteName);
            if (campaign != "")
            {
                // Log campaign
                if ((CMSContext.CurrentPageInfo != null) && AnalyticsHelper.SetCampaign(campaign, CurrentSiteName, CMSContext.CurrentPageInfo.NodeAliasPath))
                {
                    CMSContext.Campaign = campaign;
                }
            }
        }

        string useCacheItemName = this.CacheItemName;

        // Seek the attachment in cache
        if ((CacheHelper.Contains(useCacheItemName)) && (CacheHelper.CacheImageEnabled(CurrentSiteName)) && (this.ViewMode == ViewModeEnum.LiveSite))
        {
            // Get OutputFile object and process it.
            outputFile = (CMSOutputFile)CacheHelper.GetItem(useCacheItemName);
        }
        else
        {
            // Get attachment, process it and store it in cache if caching enabled.
            ProcessAttachment();

            // Save to cache if enabled and live site mode
            if (CacheHelper.CacheImageEnabled(CurrentSiteName) && (this.ViewMode == ViewModeEnum.LiveSite))
            {
                int cacheMinutes = CacheHelper.CacheImageMinutes(CurrentSiteName);

                CacheDependency cd = null;
                if (outputFile != null)
                {
                    string[] dependencies = new string[] { 
                        "node|" + CurrentSiteName.ToLower() + "|" + outputFile.AliasPath.ToLower(), 
                        "" 
                    };

                    // Do not cache if too big file which would be stored in memory
                    if (outputFile.Attachment != null)
                    {
                        if (!CacheHelper.CacheImageAllowed(CurrentSiteName, outputFile.Attachment.AttachmentSize) && !AttachmentManager.StoreFilesInFileSystem(CurrentSiteName))
                        {
                            cacheMinutes = largeFilesCacheMinutes;
                        }

                        dependencies[1] = "attachment|" + outputFile.Attachment.AttachmentGUID.ToString().ToLower();
                    }

                    cd = CacheHelper.GetCacheDependency(dependencies);
                }

                if (cacheMinutes > 0)
                {
                    CacheHelper.Add(useCacheItemName, outputFile, cd, DateTime.Now.AddMinutes(cacheMinutes), Cache.NoSlidingExpiration, CacheHelper.CacheItemPriority, null);
                }
            }
        }

        // Send the data
        SendFile(outputFile);

        ConnectionHelper.ReleaseContext();
    }


    /// <summary>
    /// Sends the given file within response
    /// </summary>
    /// <param name="file">File to send</param>
    protected void SendFile(CMSOutputFile file)
    {
        // Clear response.
        Response.Cookies.Clear();
        Response.Clear();

        Response.Cache.SetRevalidation(HttpCacheRevalidation.AllCaches);

        if ((file != null) && file.IsValid)
        {
            // Redirect if the file should be redirected
            if (file.RedirectTo != "")
            {
                // Log hit before redirecting
                if ((this.ViewMode == ViewModeEnum.LiveSite) && (file.FileNode != null) && (file.FileNode.NodeClassName.ToLower() == "cms.file"))
                {
                    if (AnalyticsHelper.AnalyticsEnabled(CurrentSiteName) && AnalyticsHelper.TrackFileDownloadsEnabled(CurrentSiteName) && !AnalyticsHelper.IsFileExtensionExcluded(CurrentSiteName, file.Attachment.AttachmentExtension) && !AnalyticsHelper.IsIPExcluded(CurrentSiteName, HttpContext.Current.Request.UserHostAddress) && !AnalyticsHelper.IsSearchEngineExcluded(CurrentSiteName))
                    {
                        HitLogProvider.LogHit(HitLogProvider.FILE_DOWNLOADS, CurrentSiteName, file.FileNode.DocumentCulture, null, file.FileNode.NodeID);
                    }
                }

                UrlHelper.Redirect(file.RedirectTo, true);
            }

            // Check authentication if secured file
            if (file.IsSecured)
            {
                URLRewriter.CheckSecured();
            }

            // Prepare etag
            string etag = file.CultureCode.ToLower();
            if (file.FileNode != null)
            {
                etag += "|" + file.FileNode.DocumentID;
            }

            if (file.IsSecured)
            {
                // For secured files, add user name to etag
                etag += "|" + HttpContext.Current.User.Identity.Name;               
            }

            // Put etag into ""
            etag = "\"" + etag + "\"";

            // Client caching - only on the live site
            if (useClientCache && (this.ViewMode == ViewModeEnum.LiveSite) && CacheHelper.CacheImageEnabled(CurrentSiteName))
            {
                // Determine the last modified date and etag sent from the browser
                string currentETag = Request.Headers["If-None-Match"];
                string ifModifiedString = Request.Headers["If-Modified-Since"];
                if ((ifModifiedString != null) && (currentETag == etag))
                {
                    // IE-browsers send something like this:
                    // If-Modified-Since: Wed, 19 Jul 2006 11:19:59 GMT; length=3350
                    DateTime ifModified;
                    if (DateTime.TryParse(ifModifiedString.Split(";".ToCharArray())[0], out ifModified))
                    {
                        // If not changed, let the browser use the cached data
                        if (file.LastModified <= ifModified.AddSeconds(1))
                        {
                            Response.StatusCode = (int)System.Net.HttpStatusCode.NotModified;
                            CompleteRequest();
                            //Response.End();
                            return;
                        }
                    }
                }
            }

            // If physical file not present, try to load
            if (file.PhysicalFile == null)
            {
                EnsurePhysicalFile(outputFile);
            }

            // If the output data should be cached, return the output data
            bool cacheOutputData = false;
            if (file.Attachment != null)
            {
                cacheOutputData = CacheHelper.CacheImageAllowed(CurrentSiteName, file.Attachment.AttachmentSize);
            }

            // Ensure the file data if physical file not present
            if (!file.DataLoaded && (file.PhysicalFile == ""))
            {
                byte[] cachedData = GetCachedOutputData();
                if (file.EnsureData(cachedData))
                {
                    if ((cachedData == null) && cacheOutputData)
                    {
                        SaveOutputDataToCache(file.OutputData, GetOutputDataDependency(file.Attachment));
                    }
                }
            }

            // Send the file
            if ((file.OutputData != null) || (file.PhysicalFile != ""))
            {
                // Setup the mime type - Fix the special types
                string mimetype = file.MimeType;
                switch (file.Attachment.AttachmentExtension.ToLower())
                {
                    case ".flv":
                        mimetype = "video/x-flv";
                        break;
                }               

                // Prepare response
                Response.ContentType = mimetype;
                SetDisposition(file.Attachment.AttachmentName, file.Attachment.AttachmentExtension);

                // Setup Etag property
                ETag = etag;

                if (useClientCache && (this.ViewMode == ViewModeEnum.LiveSite) && (CacheHelper.CacheImageEnabled(CurrentSiteName)))
                {
                    DateTime expires = DateTime.Now;

                    // Send last modified header to allow client caching
                    Response.Cache.SetLastModified(file.LastModified);
                    if (!file.IsSecured)
                    {
                        Response.Cache.SetCacheability(HttpCacheability.Public);
                        if (DocumentBase.AllowClientCache() && DocumentBase.UseFullClientCache)
                        {
                            expires = DateTime.Now.AddMinutes(CacheHelper.CacheImageMinutes(CurrentSiteName));
                        }
                    }

                    Response.Cache.SetExpires(expires);
                    Response.Cache.SetETag(etag);
                }

                // Log hit
                if ((this.ViewMode == ViewModeEnum.LiveSite) && (file.FileNode != null) && (file.FileNode.NodeClassName.ToLower() == "cms.file"))
                {
                    if (AnalyticsHelper.AnalyticsEnabled(CurrentSiteName) && AnalyticsHelper.TrackFileDownloadsEnabled(CurrentSiteName) && !AnalyticsHelper.IsFileExtensionExcluded(CurrentSiteName, file.Attachment.AttachmentExtension) && !AnalyticsHelper.IsIPExcluded(CurrentSiteName, HttpContext.Current.Request.UserHostAddress) && !AnalyticsHelper.IsSearchEngineExcluded(CurrentSiteName))
                    {
                        HitLogProvider.LogHit(HitLogProvider.FILE_DOWNLOADS, CurrentSiteName, file.FileNode.DocumentCulture, null, file.FileNode.NodeID);
                    }
                }                 

                // Add the file data
                if ((file.PhysicalFile != "") && (file.OutputData == null))
                {
                    // Stream the file from the file system
                    file.OutputData = WriteFile(file.PhysicalFile, cacheOutputData);
                }
                else
                {
                    // Use output data of the file in memory if present
                    WriteBytes(file.OutputData);
                }
            }
            else
            {
                NotFound();
            }
        }
        else
        {
            NotFound();
        }

        CompleteRequest();
        //Response.End();
    }


    /// <summary>
    /// Processes the attachment.
    /// </summary>
    protected void ProcessAttachment()
    {
        // If guid given, process the attachment
        Guid guid = QueryHelper.GetGuid("guid", Guid.Empty);
        if (guid != Guid.Empty)
        {
            // Check version
            int versionHistoryID = QueryHelper.GetInteger("versionhistoryid", 0);
            if (versionHistoryID > 0)
            {
                ProcessFile(guid, versionHistoryID);
            }
            else
            {
                ProcessFile(guid);
            }
        }
        else
        {
            // Get by node GUID
            Guid nodeguid = QueryHelper.GetGuid("nodeguid", Guid.Empty);
            if (nodeguid != Guid.Empty)
            {
                // If node GUID given, process the file
                ProcessNode(nodeguid);
            }
            else
            {
                // Get by alias path and file name
                string aliaspath = QueryHelper.GetString("aliaspath", null);
                string filename = QueryHelper.GetString("filename", null);
                if (aliaspath != null)
                {
                    ProcessNode(aliaspath, filename);
                }
            }
        }
    }


    /// <summary>
    /// Set rate if width is empty and height not or reversely
    /// </summary>
    /// <param name="pictureWidth">Image width</param>
    /// <param name="pictureHeight">Image height</param>
    private void SetRate(int pictureWidth, int pictureHeight)
    {
        if ((pictureWidth == 0) && (pictureHeight == 0))
        {
            return;
        }

        // Check the size
        if (this.Width > pictureWidth)
        {
            this.Width = pictureWidth;
        }
        if (this.Height > pictureHeight)
        {
            this.Height = pictureHeight;
        }

        float ratio = 0;

        if ((this.Width > 0) && (this.Height == 0))
        {
            ratio = (float)pictureWidth / (float)this.Width;
            this.Height = (int)((float)pictureHeight / (float)ratio);
        }

        if ((this.Width == 0) && (this.Height > 0))
        {
            ratio = (float)pictureHeight / (float)this.Height;
            this.Width = (int)((float)pictureWidth / (float)ratio);
        }
    }


    /// <summary>
    /// Processes the specified file and returns the data to the output stream
    /// </summary>
    /// <param name="attachmentGuid">Attachment guid.</param>
    protected void ProcessFile(Guid attachmentGuid)
    {
        AttachmentInfo atInfo = null;

        bool requiresData = true;
        // Check if it is necessary to load the file data
        if (useClientCache && (this.ViewMode == ViewModeEnum.LiveSite) && CacheHelper.CacheImageEnabled(CurrentSiteName))
        {
            // If possibly cached by client, do not load data (may not be sent)
            string ifModifiedString = Request.Headers["If-Modified-Since"];
            if (ifModifiedString != null)
            {
                requiresData = false;
            }
        }

        // If output data available from cache, do not require loading the data
        byte[] cachedData = GetCachedOutputData();
        if (cachedData != null)
        {
            requiresData = false;
        }

        // Get AttachmentInfo object
        if (this.ViewMode != ViewModeEnum.LiveSite)
        {
            // Not livesite mode - get latest version
            atInfo = DocumentHelper.GetAttachment(attachmentGuid, this.TreeProvider, CurrentSiteName);
        }
        else
        {
            if (!requiresData || AttachmentManager.StoreFilesInFileSystem(CurrentSiteName))
            {
                // Do not require data from DB - Not necessary or available from file system
                atInfo = this.AttachmentManager.GetAttachmentInfoWithoutBinary(attachmentGuid, CurrentSiteName);
            }
            else
            {
                // Require data from DB - Stored in DB
                atInfo = this.AttachmentManager.GetAttachmentInfo(attachmentGuid, CurrentSiteName);
            }
        }

        if (atInfo != null)
        {
            // Get the document node
            if ((node == null) && (AttachmentManager.CheckPublishedFiles(this.CurrentSiteName) || AttachmentManager.CheckFilesPermissions(this.CurrentSiteName)))
            {
                // Try to get from cache
                string nodeCacheKey = CMSContext.BaseCacheKey + "|getfile|nodebydocumentid|" + atInfo.AttachmentDocumentID.ToString();
                if ((this.CacheMinutes > 0) && CacheHelper.Contains(nodeCacheKey))
                {
                    node = (TreeNode)CacheHelper.GetItem(nodeCacheKey);
                }
                else
                {
                    // Get the document
                    node = this.TreeProvider.SelectSingleDocument(atInfo.AttachmentDocumentID, false);
                    if (node != null)
                    {
                        if (this.CacheMinutes > 0)
                        {
                            CacheDependency cd = CacheHelper.GetCacheDependency(new string[] { "node|" + this.CurrentSiteName.ToLower() + "|" + node.NodeAliasPath.ToLower() });
                            CacheHelper.Add(nodeCacheKey, node, cd, DateTime.Now.AddMinutes(this.CacheMinutes), Cache.NoSlidingExpiration);
                        }

                        if (AttachmentManager.CheckFilesPermissions(this.CurrentSiteName))
                        {
                            // Load secured values
                            node.LoadInheritedValues(new string[] { "IsSecuredNode", "RequiresSSL" });
                        }
                    }
                }
                //TreeNode tn = DocumentHelper.GetDocument(atInfo.AttachmentDocumentID, tree);
            }

            bool secured = false;
            if ((node != null) && AttachmentManager.CheckFilesPermissions(this.CurrentSiteName))
            {
                secured = (node.IsSecuredNode == 1);

                // Check secured pages
                if (secured)
                {
                    URLRewriter.CheckSecuredAreas(false);
                }
                if (node.RequiresSSL == 1)
                {
                    URLRewriter.RequestSecurePage(false, node.RequiresSSL);
                }

                // Check permissions
                bool checkPermissions = false;
                switch (URLRewriter.CheckPagePermissions(CurrentSiteName))
                {
                    case PageLocationEnum.All:
                        checkPermissions = true;
                        break;

                    case PageLocationEnum.SecuredAreas:
                        checkPermissions = secured;
                        break;
                }

                // Check the read permission for the page
                if (checkPermissions)
                {
                    if (CurrentUser.IsAuthorizedPerDocument(node, NodePermissionsEnum.Read) == AuthorizationResultEnum.Denied)
                    {
                        UrlHelper.Redirect(URLRewriter.AccessDeniedPageURL(CurrentSiteName));
                    }
                }
            }


            bool resizeImage = (ImageHelper.IsImage(atInfo.AttachmentExtension) && AttachmentManager.CanResizeImage(atInfo, this.Width, this.Height, this.MaxSideSize));

            // If the file should be redirected, redirect the file
            if ((this.ViewMode == ViewModeEnum.LiveSite) && SettingsKeyProvider.GetBoolValue(CurrentSiteName + ".CMSRedirectFilesToDisk"))
            {
                if (AttachmentManager.StoreFilesInFileSystem(CurrentSiteName))
                {
                    string path = null;
                    if (!resizeImage)
                    {
                        path = AttachmentManager.GetFilePhysicalURL(CurrentSiteName, atInfo.AttachmentGUID.ToString(), atInfo.AttachmentExtension);
                    }
                    else
                    {
                        int[] newDim = ImageHelper.EnsureImageDimensions(this.Width, this.Height, this.MaxSideSize, atInfo.AttachmentImageWidth, atInfo.AttachmentImageHeight);
                        path = AttachmentManager.GetFilePhysicalURL(CurrentSiteName, atInfo.AttachmentGUID.ToString(), atInfo.AttachmentExtension, newDim[0], newDim[1]);
                    }

                    // If path is valid, redirect
                    if (path != null)
                    {
                        // Check if file exists
                        string filePath = Server.MapPath(path);
                        if (File.Exists(filePath))
                        {
                            outputFile = new CMSOutputFile();
                            outputFile.IsSecured = secured;
                            outputFile.RedirectTo = path;
                            outputFile.Attachment = atInfo;
                        }
                    }
                }
            }

            // Get the data
            if ((outputFile == null) || (outputFile.Attachment == null))
            {
                outputFile = new CMSOutputFile(atInfo, null);
                outputFile.Width = this.Width;
                outputFile.Height = this.Height;
                outputFile.MaxSideSize = this.MaxSideSize;
                outputFile.SiteName = this.CurrentSiteName;
                outputFile.Resized = resizeImage;

                // Load the data if required
                if (requiresData)
                {
                    // Try to get the physical file
                    EnsurePhysicalFile(outputFile);
                    bool loadData = (outputFile.PhysicalFile == "");

                    // Load data if necessary
                    if (loadData)
                    {
                        if (atInfo.AttachmentBinary != null)
                        {
                            // Load from the attachment
                            outputFile.LoadData(atInfo.AttachmentBinary, this.AttachmentManager);
                        }
                        else
                        {
                            // Load from the disk
                            byte[] data = this.AttachmentManager.GetFile(atInfo, CurrentSiteName);
                            outputFile.LoadData(data, this.AttachmentManager);
                        }

                        // Save data to the cache
                        SaveOutputDataToCache(outputFile.OutputData, GetOutputDataDependency(outputFile.Attachment));
                    }
                }
                else if (cachedData != null)
                {
                    // Load the cached data if available
                    outputFile.OutputData = cachedData;
                }
            }

            if (outputFile != null)
            {
                outputFile.IsSecured = secured;

                // Add node data
                if (node != null)
                {
                    outputFile.AliasPath = node.NodeAliasPath;
                    outputFile.CultureCode = node.DocumentCulture;
                    outputFile.FileNode = node;
                    if (this.ViewMode == ViewModeEnum.LiveSite)
                    {
                        outputFile.ValidFrom = ValidationHelper.GetDateTime(node.GetValue("DocumentPublishFrom"), DateTime.MinValue);
                        outputFile.ValidTo = ValidationHelper.GetDateTime(node.GetValue("DocumentPublishTo"), DateTime.MaxValue);
                    }
                }
            }
        }
    }


    /// <summary>
    /// Processes the specified document node
    /// </summary>
    /// <param name="aliasPath">Alias path</param>
    /// <param name="fileName">File name</param>
    protected void ProcessNode(string aliasPath, string fileName)
    {
        // Load the document node
        if (node == null)
        {
            // Try to get from cache
            string nodeCacheKey = CMSContext.BaseCacheKey + "|getfile|nodebyaliaspath|" + aliasPath;
            if ((this.CacheMinutes > 0) && CacheHelper.Contains(nodeCacheKey))
            {
                node = (TreeNode)CacheHelper.GetItem(nodeCacheKey);
            }
            else
            {
                // Get the document
                if (fileName == null)
                {
                    // CMS.File
                    node = this.TreeProvider.SelectSingleNode(this.CurrentSiteName, aliasPath, CMSContext.PreferredCultureCode, SettingsKeyProvider.GetBoolValue(CurrentSiteName + ".CMSCombineImagesWithDefaultCulture"), "CMS.File", null, null, -1, false);
                }
                else
                {
                    // Other document types
                    node = this.TreeProvider.SelectSingleNode(CurrentSiteName, aliasPath, TreeProvider.ALL_CULTURES, SettingsKeyProvider.GetBoolValue(CurrentSiteName + ".CMSCombineImagesWithDefaultCulture"), null, false);
                }

                if (node != null)
                {
                    if (this.CacheMinutes > 0)
                    {
                        CacheDependency cd = CacheHelper.GetCacheDependency(new string[] { "node|" + this.CurrentSiteName.ToLower() + "|" + node.NodeAliasPath.ToLower() });
                        CacheHelper.Add(nodeCacheKey, node, cd, DateTime.Now.AddMinutes(this.CacheMinutes), Cache.NoSlidingExpiration);
                    }

                    if (AttachmentManager.CheckFilesPermissions(this.CurrentSiteName))
                    {
                        // Load secured values
                        node.LoadInheritedValues(new string[] { "IsSecuredNode", "RequiresSSL" });
                    }
                }
            }
        }

        if (node != null)
        {
            // If not published, return no content
            if ((this.ViewMode == ViewModeEnum.LiveSite) && !node.IsPublished)
            {
                outputFile = new CMSOutputFile(null, null);
                outputFile.AliasPath = node.NodeAliasPath;
                outputFile.CultureCode = node.DocumentCulture;
                outputFile.FileNode = node;
                outputFile.Height = this.Height;
                outputFile.Width = this.Width;
                outputFile.MaxSideSize = this.MaxSideSize;
            }

            // Get the document data
            if (this.ViewMode != ViewModeEnum.LiveSite)
            {
                node = DocumentHelper.GetDocument(node, this.TreeProvider);
            }

            // Get valid site name if link
            if (node.IsLink)
            {
                node = this.TreeProvider.GetOriginalNode(node);
                if (node != null)
                {
                    SiteInfo si = SiteInfoProvider.GetSiteInfo(node.NodeSiteID);
                    if (si != null)
                    {
                        CurrentSiteName = si.SiteName;
                    }
                }
            }

            // Process the node
            if (node != null)
            {
                // Get the attachment
                if (fileName == null)
                {
                    // CMS.File
                    Guid attachmentGuid = ValidationHelper.GetGuid(node.GetValue("FileAttachment"), Guid.Empty);
                    if (attachmentGuid != Guid.Empty)
                    {
                        ProcessFile(attachmentGuid);
                    }
                }
                else
                {
                    AttachmentInfo ai = null;

                    // Other document types, get the attachment by file name
                    if (this.ViewMode != ViewModeEnum.LiveSite)
                    {
                        // Not livesite mode - get latest version
                        ai = DocumentHelper.GetAttachment(node, fileName, this.TreeProvider, false);
                    }
                    else
                    {
                        // Live site mode, get directly from database
                        ai = this.AttachmentManager.GetAttachmentInfo(node.DocumentID, fileName, false);
                    }

                    if (ai != null)
                    {
                        ProcessFile(ai.AttachmentGUID);
                    }
                }
            }
        }
    }


    /// <summary>
    /// Processes the specified document node
    /// </summary>
    /// <param name="nodeGuid">Node GUID</param>
    protected void ProcessNode(Guid nodeGuid)
    {
        string columnName = QueryHelper.GetString("columnName", String.Empty);

        // Load the document node
        if (node == null)
        {
            // Try to get from cache
            string nodeCacheKey = CMSContext.BaseCacheKey + "|getfile|nodebyguid|" + nodeGuid.ToString();
            if ((this.CacheMinutes > 0) && CacheHelper.Contains(nodeCacheKey))
            {
                node = (TreeNode)CacheHelper.GetItem(nodeCacheKey);
            }
            else
            {
                // Get the document
                if (columnName == "")
                {
                    // CMS.File
                    node = this.TreeProvider.SelectSingleNode(this.CurrentSiteName, "/%", CMSContext.PreferredCultureCode, SettingsKeyProvider.GetBoolValue(CurrentSiteName + ".CMSCombineImagesWithDefaultCulture"), "CMS.File", "NodeGUID = '" + nodeGuid.ToString() + "'", null, -1, false);
                }
                else
                {
                    // Other document types
                    node = this.TreeProvider.SelectSingleNode(nodeGuid, TreeProvider.ALL_CULTURES, CurrentSiteName);
                    if (node != null)
                    {
                        // Get full record
                        node = this.TreeProvider.SelectSingleNode(this.CurrentSiteName, node.NodeAliasPath, CMSContext.PreferredCultureCode, SettingsKeyProvider.GetBoolValue(CurrentSiteName + ".CMSCombineImagesWithDefaultCulture"), null, false);
                    }
                }

                if (node != null)
                {
                    if (this.CacheMinutes > 0)
                    {
                        CacheDependency cd = CacheHelper.GetCacheDependency(new string[] { "node|" + this.CurrentSiteName.ToLower() + "|" + node.NodeAliasPath.ToLower() });
                        CacheHelper.Add(nodeCacheKey, node, cd, DateTime.Now.AddMinutes(this.CacheMinutes), Cache.NoSlidingExpiration);
                    }

                    if (AttachmentManager.CheckFilesPermissions(this.CurrentSiteName))
                    {
                        // Load secured values
                        node.LoadInheritedValues(new string[] { "IsSecuredNode", "RequiresSSL" });
                    }
                }
            }
        }

        if (node != null)
        {
            // If not published, return no content
            if ((this.ViewMode == ViewModeEnum.LiveSite) && !node.IsPublished)
            {
                outputFile = new CMSOutputFile(null, null);
                outputFile.AliasPath = node.NodeAliasPath;
                outputFile.CultureCode = node.DocumentCulture;
                outputFile.FileNode = node;
                outputFile.Height = this.Height;
                outputFile.Width = this.Width;
                outputFile.MaxSideSize = this.MaxSideSize;
            }

            // Get the document data
            if (this.ViewMode != ViewModeEnum.LiveSite)
            {
                node = DocumentHelper.GetDocument(node, this.TreeProvider);
            }

            // Get valid site name if link
            if (node.IsLink)
            {
                node = this.TreeProvider.GetOriginalNode(node);
                if (node != null)
                {
                    SiteInfo si = SiteInfoProvider.GetSiteInfo(node.NodeSiteID);
                    if (si != null)
                    {
                        CurrentSiteName = si.SiteName;
                    }
                }
            }

            // Process the node
            if (node != null)
            {
                Guid attachmentGuid = Guid.Empty;
                if (columnName != "")
                {
                    attachmentGuid = ValidationHelper.GetGuid(node.GetValue(columnName), Guid.Empty);
                }
                else
                {
                    attachmentGuid = ValidationHelper.GetGuid(node.GetValue("FileAttachment"), Guid.Empty);
                }

                if (attachmentGuid != Guid.Empty)
                {
                    ProcessFile(attachmentGuid);
                }
            }
        }
    }


    /// <summary>
    /// Processes the specified version of the file and returns the data to the output stream
    /// </summary>
    /// <param name="attachmentGuid">Attachment GUID</param>
    /// <param name="versionHistoryId">Document version history ID</param>
    protected void ProcessFile(Guid attachmentGuid, int versionHistoryId)
    {
        AttachmentInfo atInfo = GetFile(attachmentGuid, versionHistoryId);
        if (atInfo != null)
        {
            // If attachment is image, try resize
            byte[] mFile = atInfo.AttachmentBinary;
            if (mFile != null)
            {
                string mimetype = null;
                if (ImageHelper.IsImage(atInfo.AttachmentExtension))
                {
                    if (AttachmentManager.CanResizeImage(atInfo, this.Width, this.Height, this.MaxSideSize))
                    {
                        mFile = this.AttachmentManager.GetImageThumbnail(atInfo, CurrentSiteName, this.Width, this.Height, this.MaxSideSize);
                        mimetype = "image/jpeg";
                    }
                }

                if (mFile != null)
                {
                    outputFile = new CMSOutputFile(atInfo, mFile);
                }
                else
                {
                    outputFile = new CMSOutputFile();
                }
                outputFile.Height = this.Height;
                outputFile.Width = this.Width;
                outputFile.MaxSideSize = this.MaxSideSize;
                outputFile.MimeType = mimetype;
            }

            // Get the file document
            if (node == null)
            {
                node = this.TreeProvider.SelectSingleDocument(atInfo.AttachmentDocumentID);
            }
            if (node != null)
            {
                // Check secured area
                SiteInfo si = SiteInfoProvider.GetSiteInfo(node.NodeSiteID);
                if (si != null)
                {
                    if (pi == null)
                    {
                        pi = PageInfoProvider.GetPageInfo(si.SiteName, node.NodeAliasPath, node.DocumentCulture, node.DocumentUrlPath, false, this.Connection);
                    }
                    if (pi != null)
                    {
                        URLRewriter.RequestSecurePage(pi, false);
                        URLRewriter.CheckSecuredAreas(pi, false);
                    }
                }

                if ((CurrentUser.IsAuthorizedPerTreeNode(node.NodeID, NodePermissionsEnum.Read) != AuthorizationResultEnum.Denied) ||
                     (CurrentUser.IsAuthorizedPerResource("cms.content", "read")) || (node.NodeOwner == CurrentUser.UserID))
                {
                    outputFile.AliasPath = node.NodeAliasPath;
                    outputFile.CultureCode = node.DocumentCulture;
                    outputFile.FileNode = node;
                }
                else
                {
                    outputFile = null;
                }
            }
        }
    }


    /// <summary>
    /// Gets the file from version history
    /// </summary>
    /// <param name="attachmentGuid">Atachment GUID</param>
    /// <param name="versionHistoryId">Version history ID</param>
    protected AttachmentInfo GetFile(Guid attachmentGuid, int versionHistoryId)
    {
        VersionManager vm = new VersionManager(this.TreeProvider);
        IDataClass dc = vm.GetAttachmentVersion(versionHistoryId, attachmentGuid);
        if ((dc == null) || (dc.IsEmpty()))
        {
            return null;
        }
        else
        {
            return new AttachmentInfo(dc.DataRow, this.Connection);
        }
    }


    /// <summary>
    /// Returns the output data dependency based on the given attachment record
    /// </summary>
    /// <param name="ai">Attachment object</param>
    protected CacheDependency GetOutputDataDependency(AttachmentInfo ai)
    {
        return (ai == null) ? null : CacheHelper.GetCacheDependency(AttachmentManager.GetDependencyCacheKeys(ai));
    }


    /// <summary>
    /// Ensures the physical file
    /// </summary>
    /// <param name="file">Output file</param>
    public bool EnsurePhysicalFile(CMSOutputFile file)
    {
        if (file == null)
        {
            return false;
        }

        // Try to link to file system
        if ((file.Attachment != null) && (file.Attachment.AttachmentID > 0) && AttachmentManager.StoreFilesInFileSystem(file.SiteName))
        {
            string filePath = this.AttachmentManager.EnsurePhysicalFile(file.Attachment, file.SiteName);
            if (filePath != null)
            {
                if (file.Resized)
                {
                    // If resized, ensure the thumbnail file
                    if (AttachmentManager.GenerateThumbnails(file.SiteName))
                    {
                        filePath = this.AttachmentManager.EnsureThumbnailFile(file.Attachment, file.SiteName, this.Width, this.Height, this.MaxSideSize);
                        if (filePath != null)
                        {
                            // Link to the physical file
                            file.PhysicalFile = filePath;
                            return true;
                        }
                    }
                }
                else
                {
                    // Link to the physical file
                    file.PhysicalFile = filePath;
                    return false;
                }
            }
        }

        file.PhysicalFile = "";
        return false;
    }
}
